﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using Opt;

namespace md2html
{
    [Description("convert Markdown formatted text files to HTML")]
    internal class Options
    {
        private readonly List<string> _FileNames = new List<string>();
        
        [Description("when enabled, (most) bare plain URLs are auto-hyperlinked - WARNING: this is a significant deviation from the markdown spec")]
        [BooleanOption("-a")]
        [BooleanOption("--auto-hyperlink")]
        public bool AutoHyperlink
        {
            get;
            set;
        }

        [BooleanOption("-h")]
        [BooleanOption("--help")]
        [Description("show help for md2html")]
        public bool ShowHelp
        {
            get;
            set;
        }

        [Description("when enabled, RETURN becomes a literal newline - WARNING: this is a significant deviation from the markdown spec")]
        [BooleanOption("-n")]
        [BooleanOption("--new-lines")]
        public bool AutoNewLines
        {
            get;
            set;
        }

        [Description("when enabled, bold and italic require non-word characters on either side - WARNING: this is a significant deviation from the markdown spec")]
        [BooleanOption("-s")]
        [BooleanOption("--strict-bold-italic")]
        public bool StrictBoldItalic
        {
            get;
            set;
        }

        [Description("when enabled, do not link automatically link email-addresses - WARNING: this is a significant deviation from the markdown spec")]
        [BooleanOption("-@")]
        [BooleanOption("--no-link-emails")]
        public bool DontLinkEmails
        {
            get;
            set;
        }

        [Description("when enabled, problematic URL characters like [, ], (, and so forth will be encoded - WARNING: this is a significant deviation from the markdown spec")]
        [BooleanOption("-e")]
        [BooleanOption("--encode-problem-url-characters")]
        public bool EncodeProblemUrlCharacters
        {
            get;
            set;
        }

        [Description("Whether to output XHTML or plain HTML")]
        [BooleanOption("-x")]
        [BooleanOption("--xhtml")]
        public bool OutputXHTML
        {
            get;
            set;
        }

        [Arguments]
        public Collection<string> FileNames
        {
            get
            {
                return new Collection<string>(_FileNames);
            }
        }
    }
}